/** Defines a child Window of Xxy
 *  Takes responsibility of creating the view (PL) as well as the Model (DAL)
 *  Is also responsible of saving the Model if the user wants to close it
 *
 *  @author Vitus Lorenz-Meyer
 *  @version 1.0 $Rev: 261 $
 */

package Control;
import Control.Editor.*;
import util.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.text.*;

import PL.*;
import DAL.*;
import DAL.Simple.*;


class ClXxy_childWnd extends JInternalFrame implements VetoableChangeListener{

	/** The Data-Abstraction-Layer associated with this child Window
	 *  @see DAL.IDiagram */
	private IDiagram mcDAL;
	/** The Presentation-Layer associated with this child Window
	 *  @see PL.IPL */
	private ClXyWidget mcPL;
	/** Grid-Visible state of the PL*/
	private boolean mbGridVisible = true;
	/** Top Frame of Xxy */
	private static JFrame mwTopWnd;
	/** Number of open Child Windows */
	static int openFrameCount = 0;
    	/** Offset relative to last Window for new Child Windows */
    	static final int xOffset = 30, yOffset = 30;

	/** returns the localized String - only for convenience so that _("string") works within this class
	 *  @param pstrGid - the english string to localize
	 *  @return the localized version of pstrGid */
        private static String _(String pstrGid){ return gnu.gettext.GettextResource.gettext(Control.ClXxy.moResBundle,pstrGid); }

	/** Contructor which takes the SelChangedListener, the status Bar and the Top Frame
	 *  @see util.ISelChangedListener
	 *  @throws an XXYError if something went wrong while initializing
	 *  @param mpISel - Object that implements the ISelChangedListener Interface
	 *  @param ptTF - status Bar
	 *  @param pParent - Top Frame of Xxy */
	public ClXxy_childWnd(ISelChangedListener mpISel,JTextField ptTF,JFrame pParent) throws EXXYError{
		super(_("Document #") + (++openFrameCount),true,true,true,true);

			mcDAL = new ClDiagram();
			init(pParent,ptTF,mpISel);

	}

	/** Contructor which takes the SelChangedListener, the status Bar, the Top Frame and a File to open
 	 *  @see util.ISelChangedListener
 	 *  @throws an XXYError if something went wrong while initializing
	 *  @param mpISel - Object that implements the ISelChangedListener Interface
	 *  @param ptTF - status Bar
	 *  @param pParent - Top Frame of Xxy
	 *  @param poFile - File Object to open */
	public ClXxy_childWnd(ISelChangedListener mpISel,JTextField ptTF,JFrame pParent,File poFile) throws EXXYError{
		super(poFile.getName(),true,true,true,true);

			++openFrameCount;
			BufferedReader f = null;
			String sXyPicCode = null, line = null;
			try {

				f = new BufferedReader(new FileReader(poFile));
				while ((line = f.readLine()) != null) {
					sXyPicCode += line;
				}
				f.close();
			} catch (IOException ex) {
				JOptionPane.showMessageDialog(mwTopWnd,
					MessageFormat.format(_("Error while reading from {0}:\n{1}"), new Object[] { poFile.getName(),ex.getMessage() }),
					_("Error!"), JOptionPane.ERROR_MESSAGE);
				dispose();
				return;
			}
			mcDAL = new ClDiagram(sXyPicCode);
			mcDAL.setFilename(poFile.getAbsolutePath());
			init(pParent,ptTF,mpISel);

	}

	/** Contructor which takes the SelChangedListener, the status Bar, the Top Frame and a String to parse as XyPic Code
	 *  @see util.ISelChangedListener
	 *  @throws an XXYError if something went wrong while initializing
	 *  @param mpISel - Object that implements the ISelChangedListener Interface
	 *  @param ptTF - status Bar
	 *  @param pParent - Top Frame of Xxy
	 *  @param pstrXyCode - String to parse as Xy-Pic Code and display */
	public ClXxy_childWnd(ISelChangedListener mpISel,JTextField ptTF,JFrame pParent,String pstrXyCode) throws EXXYError{
		super(_("Document #") + (++openFrameCount),true,true,true,true);

			mcDAL = new ClDiagram(pstrXyCode);
			init(pParent,ptTF,mpISel);

	}

	/** Initializes this Child Window
	 *  @throws an XXYError if something went wrong while initializing
	 *  @param mpISel - Object that implements the ISelChangedListener Interface
	 *  @param ptTF - status Bar
	 *  @param pParent - Top Frame of Xxy */
	private void init(JFrame pParent,JTextField ptTF,ISelChangedListener mpISel) throws EXXYError{
		mwTopWnd = pParent;
              	this.addInternalFrameListener(
			new InternalFrameAdapter() {
				public void InternalFrameClosing(InternalFrameEvent e){
						if (mcDAL.getDirty()){
         			   			int returned = JOptionPane.showConfirmDialog( mwTopWnd, _("Do you want to save this diagram before closing it?"),
         			   				MessageFormat.format(_("Close {0}"), new Object[] { getTitle() }),
         			   				JOptionPane.YES_NO_CANCEL_OPTION);
            						if (JOptionPane.YES_OPTION == returned){
			            				try{ save(); }
			            				catch(EXXYError ei){ JOptionPane.showMessageDialog(mwTopWnd,
			            							MessageFormat.format(_("Internal Error while saving:\n{0}"), new Object[] { ei.getMessage() }),
			            							_("Internal Error!"), JOptionPane.ERROR_MESSAGE ); }
			            				catch(Exception ex){ JOptionPane.showMessageDialog(mwTopWnd,
			            							MessageFormat.format(_("Error while saving:\n{0}"), new Object[] { ex.getMessage() }),
			            							_("Error!"), JOptionPane.ERROR_MESSAGE ); }
            							dispose();
							}
            						else if (JOptionPane.NO_OPTION == returned)
            							dispose();
            					}
        				}
			}
		);

		//mcPL = new ClXyWidget(ptTF);
		mcPL = new ClXyWidget(mcDAL,ptTF);
		mcPL.setSelChangedListener(mpISel);
		mcPL.setGridVisible(mbGridVisible);

		setLocation(xOffset*openFrameCount, yOffset*openFrameCount);
		setSize(new Dimension(400,300));
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		addVetoableChangeListener(this);
		setFrameIcon(new ImageIcon("./res/intwindow.gif"));
		getContentPane().add(mcPL.getScrollPane());
		show();
	}

	/** Saves the diagram depicted in this Child Window
	 *  @throws an XXYError if something went wrong somewhere in the DAL (toString) */
	protected void save() throws EXXYError{
		String FileName = mcDAL.getFilename();
		JFileChooser fc = new JFileChooser();
		if (FileName == null){
           		fc.setDialogType( JFileChooser.SAVE_DIALOG );
        		fc.setMultiSelectionEnabled( false );
			fc.addChoosableFileFilter(new ClFileFilter("Xxy", _("Xxy files")));

            		if(fc.showSaveDialog(mwTopWnd) == JFileChooser.APPROVE_OPTION){
    				FileName = fc.getSelectedFile().getAbsolutePath();
    				mcDAL.setFilename(FileName);
    				setTitle(fc.getSelectedFile().getName());
    			}
    			else return;
    		}
    		try {
			FileWriter f1 = new FileWriter(FileName);
			f1.write(mcDAL.toString());
			f1.close();
		} catch (IOException ei) { JOptionPane.showMessageDialog(mwTopWnd,
			            		MessageFormat.format(_("Error while writing to {0}:\n{1}"), new Object[] { fc.getSelectedFile().getName(),ei.getMessage() }),
			            		_("Error!"), JOptionPane.ERROR_MESSAGE ); }

		mcDAL.setDirty(false);
	}

	/** Asks the User and saves the diagram depicted in this Child Window under a new Name
	 *  @throws an XXYError if something went wrong somewhere in the DAL (toString) */
	protected void saveAs() throws EXXYError{
		String FileName = null;
		JFileChooser fc = new JFileChooser();
            	fc.setDialogType( JFileChooser.SAVE_DIALOG );
        	fc.setMultiSelectionEnabled( false );
		fc.addChoosableFileFilter(new ClFileFilter("Xxy", _("Xxy files")));
		fc.setDialogTitle("Save As...");

		if(fc.showSaveDialog(mwTopWnd) == JFileChooser.APPROVE_OPTION){
    			FileName = fc.getSelectedFile().getAbsolutePath();
    			mcDAL.setFilename(FileName);
    			setTitle(fc.getSelectedFile().getName());
    		}
    		else return;

    		try {
			FileWriter f1 = new FileWriter(FileName);
			f1.write(mcDAL.toString());
			f1.close();
		} catch (IOException ei) {  JOptionPane.showMessageDialog(mwTopWnd,
			            		MessageFormat.format(_("Error while writing to {0}:\n{1}"), new Object[] { fc.getSelectedFile().getName(),ei.getMessage() }),
			            		_("Error!"), JOptionPane.ERROR_MESSAGE ); }

		mcDAL.setDirty(false);
	}
	/** Sets the Grid Visible Property of the PL
	 *  @throws an XXYError if something went wrong somewhere in the PL
	 *  @see PL.IPL */
	protected void setGridVisible(boolean pbGrid) throws EXXYError{
        	mbGridVisible=pbGrid;
        	mcPL.setGridVisible(pbGrid);

        }
        /** Gets the Grid Visible Property of the PL
         *  @throws an XXYError if something went wrong somewhere in the PL
	 *  @return the grid visible property of the PL
	 *  @see PL.IPL */
	protected boolean getGridVisible(){
        	return mbGridVisible;
        }
        /** Gets the Grid Size Property of the PL
         *  @return the grid size of the PL
         *  @see PL.IPL */
	protected int getGridSize(){
        	return mcPL.getGridSize();
        }
        /** Sets the Grid Size Property of the PL
         *  @param pGS - requested Grid Size
         *  @throws an XXYError if something went wrong somewhere in the PL
	 *  @see PL.IPL */
	protected void setGridSize(int pGS) throws EXXYError{
        	mcPL.setGridSize(pGS);
       }
	/** Deletes in the PL selected Object
	 *  @throws an XXYError if something went wrong somewhere in the PL
	 *  @see PL.IPL */
	protected void deleteSelected() throws EXXYError{
		mcPL.deleteSelected();
		mcPL.repaint();
	}
	/** Notifies the PL of a tool change
	 *  @param ps8Tool - newly selected Tool
         *  @throws an XXYError if something went wrong somewhere in the PL
	 *  @see PL.IPL */
	protected void setTool(byte ps8Tool) throws EXXYError{
		mcPL.setTool(ps8Tool);
	}
	/** Refreshes the Language (title) and the EditorWidget of this InternalFrame
	 *  @param mpISel - new ISelChangedListener */
	protected void refresh(ISelChangedListener mpISel){
		//refresh the title if it says "unnamed #" ...
		if (mcDAL.getFilename().equals("")){
			String tit = getTitle();
			setTitle(_("Document #") + tit.substring(tit.length()-1,tit.length()));
			//setTitle("ficken");
		}
		//refresh Editor Widget association
		mcPL.setSelChangedListener(mpISel);

		repaint();
		show();
	}
	/** returns the Xy-Pic Code String representation of this Diagram
	 *  @throws an XXYError if something went wrong somewhere in the DAL
	 *  @return the String representation of this Diagram
	 *  @see DAL.IDiagram */
	public String XyString() throws EXXYError{
		return mcDAL.toString();
	}
	/** Closes this Child Window (overriden from Component) */
	public void dispose(){
		openFrameCount--;
		super.dispose();
	}
	/** repaints this Child Window (overriden from Component) */
	public void repaint(){
		try{ mcPL.repaint(); }
		catch(Exception e){ }
		super.repaint();
	}
	/** Gets Called when this Child Window is about to get closed, enables it to veto closing
	 *  @param pce - the Event of the changed Property
         *  @throws an PropertyVetoException if the user canceled
	 *  @see VetoableChangeListener */
	public void vetoableChange(PropertyChangeEvent pce) throws PropertyVetoException {
		if (pce.getPropertyName().equals(IS_CLOSED_PROPERTY)) {
			boolean changed = ((Boolean) pce.getNewValue()).booleanValue();
			if (changed) {
				//the closed-state of this Internal Frame has changed => is about to be closed
				if (mcDAL.getDirty()){
					//unsaved changes
         				int returned = JOptionPane.showConfirmDialog( mwTopWnd, _("Do you want to save this diagram before closing it?"),
         			   				MessageFormat.format(_("Close {0}"), new Object[] { getTitle() }),
         			   				JOptionPane.YES_NO_CANCEL_OPTION);
            				if (JOptionPane.YES_OPTION == returned){
			            		try{ save(); }
			            		catch(EXXYError ei){ JOptionPane.showMessageDialog(mwTopWnd,
			            					MessageFormat.format(_("Internal Error while saving:\n{0}"), new Object[] { ei.getMessage() }),
			            					_("Internal Error!"), JOptionPane.ERROR_MESSAGE ); }
			            		catch(Exception ex){ JOptionPane.showMessageDialog(mwTopWnd,
			            					MessageFormat.format(_("Error while saving:\n{0}"), new Object[] { ex.getMessage() }),
			            					_("Error!"), JOptionPane.ERROR_MESSAGE ); }
            				}
            				else if (JOptionPane.NO_OPTION == returned)
            					//user selected "no"
            					dispose();
            				//user selected "cancel"
            				else throw new PropertyVetoException("Cancelled",null);
				}
            		}
      		}
  	}


}
